﻿'
'   NumericUpDown Mouse-Wheel (1カウント対応版)
'
'   (注)
'       マウスホイールの1目盛りでスクロールする行数の設定が
'       120を整数で割れる数( 1～6,8,10,12,… ) 以外はカウントに不具合
'       (端数発生時にカウントしない)が発生します。
'
'       Hexadecimal が 'True' の時は、Minimum の設定値を負(-)にしないで下さい。
'
'       下記仕様の為、上位桁が破棄されてしまい値が変わってしまいます。
'       例えば、最小値: -100, 最大値: 100 の場合、100は 0x64 ですので最大桁数は 2桁になります。
'
'       -1の時、実際の値は 0xFFFFFFFFFFFFFFFF ですが、下位2桁以外破棄されますので 0xFF(255)
'       となり不整合が生じます。
'
'   (仕様)
'       マウスホイールで数値を変更中は、カーソルを隠します。左クリックでカーソルが表示されます。
'       テキストボックス内で左クリックするとカーソルが表示されます。
'
'       貼り付けによる入力時は、先頭より符号を除いた最大値の桁数を対象に文字を取り込み、
'       指定範囲内の数値に変換（内部既定処理）
'
'       直接入力時は以下のルールを適用
'       ・最大桁数以下の場合は単純挿入
'       ・最大桁の場合はカーソルのある位置に挿入し、最大桁より右側(下位)の文字は破棄

Imports System.Globalization
Imports System.Text.RegularExpressions

Namespace UserLib

<DebuggerStepThrough()>
Public Class NumericUpDownEx
    Inherits NumericUpDown

    Private Declare Function ShowCaret Lib "user32.dll" Alias "ShowCaret" (ByVal hwnd As IntPtr) As Boolean
    Private Declare Function HideCaret Lib "user32.dll" Alias "HideCaret" (ByVal hwnd As IntPtr) As Boolean

    Private _TextBoxCtl As Control
    Private _CaretState As Boolean = True
    Private _WheelDelta As Integer = 0
    Private _TextValue As Decimal

    Protected Overrides Sub WndProc(ByRef m As Message)
        If m.Msg = &H20A Then                                   'WM_MOUSEWHEEL
           If SystemInformation.MouseWheelScrollLines > 0 Then
              Dim delta As Integer
              _WheelDelta += (m.WParam.ToInt32 >> 16)           '16bit Right-Shift
              delta = (_WheelDelta \ 120) * 120 \ SystemInformation.MouseWheelScrollLines
              _WheelDelta = _WheelDelta Mod 120
              m.WParam = New IntPtr((delta << 16) Or (m.WParam.ToInt32 And &HFFFF))
           End If
        End If
        MyBase.WndProc(m)
    End Sub

    Protected Overrides Sub OnGotFocus(ByVal e As EventArgs)
        _CaretState = True
        MyBase.OnGotFocus(e)
    End Sub

    Protected Overrides Sub OnLeave(ByVal e As EventArgs)
        _CaretState = True
        MyBase.OnLeave(e)
    End Sub

    Protected Overrides Sub OnClick(ByVal e As EventArgs)
        If Not _CaretState Then
           _CaretState = True

           For Each ctl As Control In Me.Controls
             If TypeOf ctl Is TextBox Then
                ShowCaret(ctl.Handle)
             End If
           Next
        End If
        MyBase.OnClick(e)
    End Sub

    Protected Overrides Sub OnMouseWheel(ByVal e As MouseEventArgs)
        If _CaretState Then
           _CaretState = False

           For Each ctl As Control In Me.Controls
             If TypeOf ctl Is TextBox Then
                HideCaret(ctl.Handle)
             End If
           Next
        End If

        MyBase.OnMouseWheel(e)
    End Sub

    Protected Overrides Sub OnTextBoxKeyPress(ByVal source As Object, ByVal e As KeyPressEventArgs)

        For Each ctl As Control In Me.Controls
          If TypeOf ctl Is TextBox Then
             _TextBoxCtl = ctl
          End If
        Next

        If Me.Hexadecimal Then

        '0-9,a-f,A-F,[BS] 以外、KeyPressイベントをキャンセル

           If Not Regex.IsMatch(e.KeyChar.ToString, "[0-9a-fA-F\b]") Then
              e.Handled = True
              Beep()
           End If
        Else

        '-(先頭のみ),0-9,[BS] 以外、KeyPressイベントをキャンセル

           If Not Regex.IsMatch(e.KeyChar.ToString, "[0-9\b]") Then
              If Not (e.KeyChar = "-"c AndAlso DirectCast(_TextBoxCtl, TextBox).SelectionStart = 0) Then
                 e.Handled = True
                 Beep()
              End If
           End If
        End If

        MyBase.OnTextBoxKeyPress(source, e)
    End Sub

    Protected Overrides Sub OnTextBoxTextChanged(ByVal source As Object, ByVal e As EventArgs)

        For Each ctl As Control In Me.Controls
          If TypeOf ctl Is TextBox Then
             _TextBoxCtl = ctl
          End If
        Next

        Dim t As TextBox = DirectCast(_TextBoxCtl, TextBox)
        Dim p As Integer = t.SelectionStart

        If Me.Hexadecimal Then
           Dim l As Integer = Convert.ToInt64(Me.Maximum).ToString("X").Length
           Dim x As New Regex(String.Format("0*([0-9a-fA-F]{{1,{0}}})", l))
           Dim m As MatchCollection = x.Matches(t.Text)

           If m.Count <> 0 Then
              t.Text = m(0).Groups(1).Value.ToUpper
           Else
              t.Text = ""
           End If
        Else
           If Not (t.TextLength = 1 AndAlso t.Text.StartsWith("-")) Then
              Dim x As New Regex("-?0*([0-9]+)")
              Dim m As MatchCollection = x.Matches(t.Text)

              If m.Count <> 0 Then
                 Dim l As Integer

              '符号を除いた最大桁数を設定
                 If Math.Abs(Me.Maximum) < Math.Abs(Me.Minimum) Then
                    l = Math.Abs(Me.Minimum).ToString.Length
                 Else
                    l = Math.Abs(Me.Maximum).ToString.Length
                 End If

              '取り込み桁数に修正
                 If l > m(0).Groups(1).Value.Length Then l = m(0).Groups(1).Value.Length

              '符号を除いた最大桁数を数値に変換
                 Dim v As Decimal = Convert.ToDecimal(m(0).Groups(1).Value.Substring(0, l))

              '符号を戻す
                 If m(0).Groups(0).Value.StartsWith("-") Then v = -v

                 t.Text = v.ToString
              Else
                 t.Text = ""
              End If
           End If
        End If

        t.SelectionStart = p

        MyBase.OnTextBoxTextChanged(source, e)
    End Sub

End Class

End Namespace
